# CSC 111, Lab 9 starter-code
# Author: Sara Mathieson (modified from Lab 8 solution)
# Author: <add your names here>
# A program to draw a fractal tree using recursion.


from graphics import *
import math
import random


#---------
# GLOBALS
#---------


theta = 0.7 # angle of each branch away from its trunk
petal_color = "purple" # color of the petals
center_color = "yellow" # color of the center of the flower
sky_color = "lightblue1" # color of the sky/background
ground_color = "dark turquoise" # color of the ground
branch_color = "navy" # color of the branches


#---------
# CLASSES
#---------


# TODO: create Flower class here


#------------------
# HELPER FUNCTIONS
#------------------


def draw_flower(window,x,y):
    """Draw a flower at the given (x,y) position."""

    # set petal length and pick a random orientation for the flower
    petal_len = 18
    angle = random.uniform(0,2*math.pi)

    # make four petals using helper function
    petal1 = make_petal(x,y,petal_len,angle)
    petal2 = make_petal(x,y,petal_len,angle+math.pi/2)
    petal3 = make_petal(x,y,petal_len,angle+math.pi)
    petal4 = make_petal(x,y,petal_len,angle+3*math.pi/2)

    # draw the four petals
    petal1.draw(window)
    petal2.draw(window)
    petal3.draw(window)
    petal4.draw(window)

    # make a yellow center for the flower
    center = Circle(Point(x,y),2)
    center.setFill(center_color)
    center.setOutline(center_color)
    center.draw(window)


def make_petal(x,y,length,angle):
    """Return a diamond shaped petal at the given (x,y) position."""

    # first point closest to the center
    p1 = Point(x,y)

    # left-most point
    dx2 = length/2*math.cos(angle-0.3)
    dy2 = length/2*math.sin(angle-0.3)
    p2 = Point(x+dx2,y+dy2)

    # furthest point from the center
    dx3 = length*math.cos(angle)
    dy3 = length*math.sin(angle)
    p3 = Point(x+dx3,y+dy3)

    # right-most point
    dx4 = length/2*math.cos(angle+0.3)
    dy4 = length/2*math.sin(angle+0.3)
    p4 = Point(x+dx4,y+dy4)

    # create the diamond-shaped petal
    petal = Polygon(p1,p2,p3,p4)
    petal.setFill(petal_color)
    petal.setOutline(petal_color)
    return petal
    

def draw_tree(window, order, x, y, length, angle):
    """Recursive function to draw the fractal tree.
    window: type GraphWin, the window on which to draw the tree
    order: type int, the level of recursion (starts high and decreases to 0)
    x,y: type int, the position of the base of this branch
    length: the length of this branch
    angle: the angle of the direction of this branch
    """

    # compute the coordinates of end of the current branch
    dx = length * math.cos(angle)
    dy = length * math.sin(angle)
    x2 = x + dx
    y2 = y + dy

    # draw current branch
    branch = Line(Point(x,y), Point(x2,y2))
    branch.setFill(branch_color)
    branch.setWidth(order) # make the higher order branches thicker
    branch.draw(window)

    # base case: at the leaves, draw a flower
    if order == 0:
        draw_flower(window, x2, y2)

    # recursion case: if order > 0, draw two subtrees
    else:
        new_len = length*0.7
        draw_tree(window, order-1, x2, y2, new_len, angle-theta) # "left" branch
        draw_tree(window, order-1, x2, y2, new_len, angle+theta) # "right" branch


#------
# MAIN
#------


def main():
    """Draw one tree in the middle of the screen, pointing straight up."""

    # set up the graphics window
    width = 800
    height = 600
    win = GraphWin("Fractal Tree", width, height)
    win.setBackground(sky_color)

    # set up a rectangle for the ground
    ground = Rectangle(Point(0,height-60), Point(width,height))
    ground.setFill(ground_color)
    ground.setOutline(ground_color)
    ground.draw(win)

    # set up parameters for our call to the recursive function
    order = 7
    x_start = width/2   # middle of the window
    y_start = height-10 # close to the bottom of the window
    angle = -math.pi/2  # initial branch (trunk) pointing *upwards*
    length = 200        # length of initial branch (trunk)

    # call our recursive function
    draw_tree(win, order, x_start, y_start, length, angle)

    # close on click
    win.getMouse()
    win.close()


main()
